/*
 *  vbftest.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include "vbftest.h"
#include <am/vbfamprojectmanager.h>
#include <vbfiprojectmanager.h>
#include <vbfproject.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <vbfmodule.h>
#include <vbfpackage.h>
#include <vbfconfignode.h>
#include <vbfgroup.h>
#include <vbftarget.h>
#include <vbfsource.h>
#include <vbffile.h>
#include <vbfvariable.h>
#include <stdio.h>
#include <glib/gi18n-lib.h>
#include <gobject/gvaluecollector.h>




enum  {
	VBF_TESTS_OPEN_TEST_DUMMY_PROPERTY
};
static char** vbf_tests_open_test_projects = NULL;
static gint vbf_tests_open_test_projects_length1 = 0;
static gint vbf_tests_open_test_projects_size = 0;
static gboolean vbf_tests_open_test_dump_variables = FALSE;
static void vbf_tests_open_test_dump_project (VbfTestsOpenTest* self, VbfProject* project);
static gpointer vbf_tests_open_test_parent_class = NULL;
static void vbf_tests_open_test_finalize (VbfTestsOpenTest* obj);

static const GOptionEntry VBF_TESTS_OPEN_TEST_options[] = {{"dump-variables", 'd', 0, G_OPTION_ARG_INT, &vbf_tests_open_test_dump_variables, "Dump variables", "0 or 1"}, {"", (gchar) 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &vbf_tests_open_test_projects, "Projects DIR", "PROJECT_DIR"}, {NULL}};


gboolean vbf_tests_open_test_run (VbfTestsOpenTest* self, const char* project_name) {
	VbfIProjectManager* pm;
	gboolean res;
	const char* _tmp0;
	gboolean _tmp3;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project_name != NULL, FALSE);
	pm = (VbfIProjectManager*) vbf_am_project_manager_new ();
	g_print ("Probing directory '%s' ...", project_name);
	res = vbf_iproject_manager_probe (pm, project_name);
	_tmp0 = NULL;
	if (res) {
		_tmp0 = "OK";
	} else {
		_tmp0 = "KO";
	}
	g_print ("%s\n", _tmp0);
	if (res) {
		VbfProject* project;
		g_print ("Opening project\n");
		project = vbf_iproject_manager_open (pm, project_name);
		if (project != NULL) {
			gboolean _tmp1;
			g_print ("Dumping...\n");
			vbf_tests_open_test_dump_project (self, project);
			return (_tmp1 = TRUE, (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), (pm == NULL) ? NULL : (pm = (g_object_unref (pm), NULL)), _tmp1);
		} else {
			gboolean _tmp2;
			g_print ("Error\n");
			return (_tmp2 = FALSE, (project == NULL) ? NULL : (project = (g_object_unref (project), NULL)), (pm == NULL) ? NULL : (pm = (g_object_unref (pm), NULL)), _tmp2);
		}
		(project == NULL) ? NULL : (project = (g_object_unref (project), NULL));
	}
	return (_tmp3 = FALSE, (pm == NULL) ? NULL : (pm = (g_object_unref (pm), NULL)), _tmp3);
}


static void vbf_tests_open_test_dump_project (VbfTestsOpenTest* self, VbfProject* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	g_print ("PROJECT:\n");
	g_print ("  name.... %s\n", project->name);
	g_print ("  version. %s\n", project->version);
	g_print ("  url..... %s\n", project->url);
	g_print ("  MODULES\n");
	{
		GeeList* _tmp0;
		GeeIterator* _tmp1;
		GeeIterator* _module_it;
		_tmp0 = NULL;
		_tmp1 = NULL;
		_module_it = (_tmp1 = gee_iterable_iterator ((GeeIterable*) (_tmp0 = vbf_project_get_modules (project))), (_tmp0 == NULL) ? NULL : (_tmp0 = (gee_collection_object_unref (_tmp0), NULL)), _tmp1);
		while (gee_iterator_next (_module_it)) {
			VbfModule* module;
			module = (VbfModule*) gee_iterator_get (_module_it);
			g_print ("    MODULE\n");
			g_print ("      name %s\n", module->name);
			g_print ("      PACKAGES\n");
			{
				GeeList* _tmp2;
				GeeIterator* _tmp3;
				GeeIterator* _package_it;
				_tmp2 = NULL;
				_tmp3 = NULL;
				_package_it = (_tmp3 = gee_iterable_iterator ((GeeIterable*) (_tmp2 = vbf_module_get_packages (module))), (_tmp2 == NULL) ? NULL : (_tmp2 = (gee_collection_object_unref (_tmp2), NULL)), _tmp3);
				while (gee_iterator_next (_package_it)) {
					VbfPackage* package;
					char* _tmp4;
					package = (VbfPackage*) gee_iterator_get (_package_it);
					g_print ("        PACKAGE\n");
					g_print ("          name....... %s\n", package->name);
					g_print ("          constraint. %s\n", package->constraint);
					_tmp4 = NULL;
					g_print ("          version.... %s\n", _tmp4 = vbf_config_node_to_string (package->version));
					_tmp4 = (g_free (_tmp4), NULL);
					(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			(module == NULL) ? NULL : (module = (g_object_unref (module), NULL));
		}
		(_module_it == NULL) ? NULL : (_module_it = (gee_collection_object_unref (_module_it), NULL));
	}
	g_print ("  GROUPS\n");
	{
		GeeList* _tmp5;
		GeeIterator* _tmp6;
		GeeIterator* _group_it;
		_tmp5 = NULL;
		_tmp6 = NULL;
		_group_it = (_tmp6 = gee_iterable_iterator ((GeeIterable*) (_tmp5 = vbf_project_get_groups (project))), (_tmp5 == NULL) ? NULL : (_tmp5 = (gee_collection_object_unref (_tmp5), NULL)), _tmp6);
		while (gee_iterator_next (_group_it)) {
			VbfGroup* group;
			group = (VbfGroup*) gee_iterator_get (_group_it);
			g_print ("    GROUP\n");
			g_print ("      name %s\n", group->name);
			g_print ("      TARGETS\n");
			{
				GeeList* _tmp7;
				GeeIterator* _tmp8;
				GeeIterator* _target_it;
				_tmp7 = NULL;
				_tmp8 = NULL;
				_target_it = (_tmp8 = gee_iterable_iterator ((GeeIterable*) (_tmp7 = vbf_group_get_targets (group))), (_tmp7 == NULL) ? NULL : (_tmp7 = (gee_collection_object_unref (_tmp7), NULL)), _tmp8);
				while (gee_iterator_next (_target_it)) {
					VbfTarget* target;
					target = (VbfTarget*) gee_iterator_get (_target_it);
					g_print ("        TARGET\n");
					g_print ("          name %s\n", target->name);
					g_print ("          SOURCES\n");
					{
						GeeList* _tmp9;
						GeeIterator* _tmp10;
						GeeIterator* _source_it;
						_tmp9 = NULL;
						_tmp10 = NULL;
						_source_it = (_tmp10 = gee_iterable_iterator ((GeeIterable*) (_tmp9 = vbf_target_get_sources (target))), (_tmp9 == NULL) ? NULL : (_tmp9 = (gee_collection_object_unref (_tmp9), NULL)), _tmp10);
						while (gee_iterator_next (_source_it)) {
							VbfSource* source;
							source = (VbfSource*) gee_iterator_get (_source_it);
							g_print ("            SOURCE filename %s\n", ((VbfFile*) source)->filename);
							(source == NULL) ? NULL : (source = (g_object_unref (source), NULL));
						}
						(_source_it == NULL) ? NULL : (_source_it = (gee_collection_object_unref (_source_it), NULL));
					}
					g_print ("          OTHER FILES\n");
					{
						GeeList* _tmp11;
						GeeIterator* _tmp12;
						GeeIterator* _file_it;
						_tmp11 = NULL;
						_tmp12 = NULL;
						_file_it = (_tmp12 = gee_iterable_iterator ((GeeIterable*) (_tmp11 = vbf_target_get_files (target))), (_tmp11 == NULL) ? NULL : (_tmp11 = (gee_collection_object_unref (_tmp11), NULL)), _tmp12);
						while (gee_iterator_next (_file_it)) {
							VbfFile* file;
							file = (VbfFile*) gee_iterator_get (_file_it);
							g_print ("            FILES filename %s\n", file->filename);
							(file == NULL) ? NULL : (file = (g_object_unref (file), NULL));
						}
						(_file_it == NULL) ? NULL : (_file_it = (gee_collection_object_unref (_file_it), NULL));
					}
					g_print ("          REFERENCED PACKAGES\n");
					{
						GeeList* _tmp13;
						GeeIterator* _tmp14;
						GeeIterator* _package_it;
						_tmp13 = NULL;
						_tmp14 = NULL;
						_package_it = (_tmp14 = gee_iterable_iterator ((GeeIterable*) (_tmp13 = vbf_target_get_packages (target))), (_tmp13 == NULL) ? NULL : (_tmp13 = (gee_collection_object_unref (_tmp13), NULL)), _tmp14);
						while (gee_iterator_next (_package_it)) {
							VbfPackage* package;
							package = (VbfPackage*) gee_iterator_get (_package_it);
							g_print ("            name....... %s\n", package->name);
							(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
						}
						(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
					}
					(target == NULL) ? NULL : (target = (g_object_unref (target), NULL));
				}
				(_target_it == NULL) ? NULL : (_target_it = (gee_collection_object_unref (_target_it), NULL));
			}
			g_print ("      REFERENCED PACKAGES\n");
			{
				GeeList* _tmp15;
				GeeIterator* _tmp16;
				GeeIterator* _package_it;
				_tmp15 = NULL;
				_tmp16 = NULL;
				_package_it = (_tmp16 = gee_iterable_iterator ((GeeIterable*) (_tmp15 = vbf_group_get_packages (group))), (_tmp15 == NULL) ? NULL : (_tmp15 = (gee_collection_object_unref (_tmp15), NULL)), _tmp16);
				while (gee_iterator_next (_package_it)) {
					VbfPackage* package;
					package = (VbfPackage*) gee_iterator_get (_package_it);
					g_print ("        PACAKGE: %s", package->name);
					g_print ("\n");
					(package == NULL) ? NULL : (package = (g_object_unref (package), NULL));
				}
				(_package_it == NULL) ? NULL : (_package_it = (gee_collection_object_unref (_package_it), NULL));
			}
			if (vbf_tests_open_test_dump_variables) {
				g_print ("      GROUP VARIABLES\n");
				{
					GeeList* _tmp17;
					GeeIterator* _tmp18;
					GeeIterator* _variable_it;
					_tmp17 = NULL;
					_tmp18 = NULL;
					_variable_it = (_tmp18 = gee_iterable_iterator ((GeeIterable*) (_tmp17 = vbf_group_get_variables (group))), (_tmp17 == NULL) ? NULL : (_tmp17 = (gee_collection_object_unref (_tmp17), NULL)), _tmp18);
					while (gee_iterator_next (_variable_it)) {
						VbfVariable* variable;
						char* _tmp19;
						variable = (VbfVariable*) gee_iterator_get (_variable_it);
						_tmp19 = NULL;
						g_print ("        VARIABLE: %s", _tmp19 = vbf_config_node_to_string ((VbfConfigNode*) variable));
						_tmp19 = (g_free (_tmp19), NULL);
						g_print ("\n");
						(variable == NULL) ? NULL : (variable = (g_object_unref (variable), NULL));
					}
					(_variable_it == NULL) ? NULL : (_variable_it = (gee_collection_object_unref (_variable_it), NULL));
				}
			}
			(group == NULL) ? NULL : (group = (g_object_unref (group), NULL));
		}
		(_group_it == NULL) ? NULL : (_group_it = (gee_collection_object_unref (_group_it), NULL));
	}
	if (vbf_tests_open_test_dump_variables) {
		g_print ("  GLOBAL VARIABLES\n");
		{
			GeeList* _tmp20;
			GeeIterator* _tmp21;
			GeeIterator* _variable_it;
			_tmp20 = NULL;
			_tmp21 = NULL;
			_variable_it = (_tmp21 = gee_iterable_iterator ((GeeIterable*) (_tmp20 = vbf_project_get_variables (project))), (_tmp20 == NULL) ? NULL : (_tmp20 = (gee_collection_object_unref (_tmp20), NULL)), _tmp21);
			while (gee_iterator_next (_variable_it)) {
				VbfVariable* variable;
				char* _tmp22;
				variable = (VbfVariable*) gee_iterator_get (_variable_it);
				_tmp22 = NULL;
				g_print ("    VARIABLE: %s", _tmp22 = vbf_config_node_to_string ((VbfConfigNode*) variable));
				_tmp22 = (g_free (_tmp22), NULL);
				g_print ("\n");
				(variable == NULL) ? NULL : (variable = (g_object_unref (variable), NULL));
			}
			(_variable_it == NULL) ? NULL : (_variable_it = (gee_collection_object_unref (_variable_it), NULL));
		}
	}
}


gint vbf_tests_open_test_main (char** args, int args_length1) {
	GError * inner_error;
	VbfTestsOpenTest* test;
	gint _tmp1;
	gint _tmp2;
	inner_error = NULL;
	{
		GOptionContext* opt_context;
		opt_context = g_option_context_new ("- Vala Build Framework Utility");
		g_option_context_set_help_enabled (opt_context, TRUE);
		g_option_context_add_main_entries (opt_context, VBF_TESTS_OPEN_TEST_options, NULL);
		g_option_context_parse (opt_context, &args_length1, &args, &inner_error);
		if (inner_error != NULL) {
			(opt_context == NULL) ? NULL : (opt_context = (g_option_context_free (opt_context), NULL));
			if (inner_error->domain == G_OPTION_ERROR) {
				goto __catch0_g_option_error;
			}
			goto __finally0;
		}
		(opt_context == NULL) ? NULL : (opt_context = (g_option_context_free (opt_context), NULL));
	}
	goto __finally0;
	__catch0_g_option_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			gint _tmp0;
			fprintf (stdout, "%s\n", e->message);
			fprintf (stdout, _ ("Run '%s --help' to see a full list of available command line options.\n"), args[0]);
			return (_tmp0 = 1, (e == NULL) ? NULL : (e = (g_error_free (e), NULL)), _tmp0);
		}
	}
	__finally0:
	if (inner_error != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
		g_clear_error (&inner_error);
		return 0;
	}
	test = vbf_tests_open_test_new ();
	_tmp1 = 0;
	if (vbf_tests_open_test_run (test, vbf_tests_open_test_projects[0])) {
		_tmp1 = 0;
	} else {
		_tmp1 = 1;
	}
	return (_tmp2 = _tmp1, (test == NULL) ? NULL : (test = (vbf_tests_open_test_unref (test), NULL)), _tmp2);
}


int main (int argc, char ** argv) {
	g_type_init ();
	return vbf_tests_open_test_main (argv, argc);
}


VbfTestsOpenTest* vbf_tests_open_test_construct (GType object_type) {
	VbfTestsOpenTest* self;
	self = (VbfTestsOpenTest*) g_type_create_instance (object_type);
	return self;
}


VbfTestsOpenTest* vbf_tests_open_test_new (void) {
	return vbf_tests_open_test_construct (VBF_TESTS_TYPE_OPEN_TEST);
}


static void vbf_tests_value_open_test_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vbf_tests_value_open_test_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vbf_tests_open_test_unref (value->data[0].v_pointer);
	}
}


static void vbf_tests_value_open_test_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vbf_tests_open_test_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vbf_tests_value_open_test_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vbf_tests_value_open_test_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		VbfTestsOpenTest* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vbf_tests_open_test_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vbf_tests_value_open_test_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	VbfTestsOpenTest** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vbf_tests_open_test_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vbf_tests_param_spec_open_test (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	VbfTestsParamSpecOpenTest* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VBF_TESTS_TYPE_OPEN_TEST), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vbf_tests_value_get_open_test (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VBF_TESTS_TYPE_OPEN_TEST), NULL);
	return value->data[0].v_pointer;
}


void vbf_tests_value_set_open_test (GValue* value, gpointer v_object) {
	VbfTestsOpenTest* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VBF_TESTS_TYPE_OPEN_TEST));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VBF_TESTS_TYPE_OPEN_TEST));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vbf_tests_open_test_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vbf_tests_open_test_unref (old);
	}
}


static void vbf_tests_open_test_class_init (VbfTestsOpenTestClass * klass) {
	vbf_tests_open_test_parent_class = g_type_class_peek_parent (klass);
	VBF_TESTS_OPEN_TEST_CLASS (klass)->finalize = vbf_tests_open_test_finalize;
}


static void vbf_tests_open_test_instance_init (VbfTestsOpenTest * self) {
	self->ref_count = 1;
}


static void vbf_tests_open_test_finalize (VbfTestsOpenTest* obj) {
	VbfTestsOpenTest * self;
	self = VBF_TESTS_OPEN_TEST (obj);
}


GType vbf_tests_open_test_get_type (void) {
	static GType vbf_tests_open_test_type_id = 0;
	if (vbf_tests_open_test_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vbf_tests_value_open_test_init, vbf_tests_value_open_test_free_value, vbf_tests_value_open_test_copy_value, vbf_tests_value_open_test_peek_pointer, "p", vbf_tests_value_open_test_collect_value, "p", vbf_tests_value_open_test_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (VbfTestsOpenTestClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_tests_open_test_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfTestsOpenTest), 0, (GInstanceInitFunc) vbf_tests_open_test_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vbf_tests_open_test_type_id = g_type_register_fundamental (g_type_fundamental_next (), "VbfTestsOpenTest", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return vbf_tests_open_test_type_id;
}


gpointer vbf_tests_open_test_ref (gpointer instance) {
	VbfTestsOpenTest* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vbf_tests_open_test_unref (gpointer instance) {
	VbfTestsOpenTest* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VBF_TESTS_OPEN_TEST_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




